from urllib.parse import urljoin
import logging

import requests
from django.http import Http404
from django.conf import settings


class TamaraPayment:
    def __init__(self):
        url = getattr(settings, "TAMARA_EXTENSION_URL", None)
        if not url:
            logging.exception("Missing TAMARA_EXTENSION_URL")
            raise Http404
        self.base_url = url

    def _send_request(self, url, method, **kwargs):
        try:
            response = requests.request(method, url, **kwargs)
            response.raise_for_status()
            return response
        except requests.exceptions.RequestException as ex:
            response_text = ex.response.text if ex.response is not None else None
            status_code = ex.response.status_code if ex.response is not None else None

            logging.exception("Tamara Payment Request Error",
                              extra={"url": url,
                                     "method": method,
                                     "status_code": status_code,
                                     "response_text": response_text,
                                     "exception": ex
                                     })

    def check_availability(self, request_body):
        url = urljoin(self.base_url, "check-availability", )
        response = self._send_request(url,
                                      "POST",
                                      json=request_body,
                                      headers={"Content-Type": "application/json"})
        if response:
            return response.json()["has_availability"]
        return False

    @property
    def url(self):
        return self.base_url



from django.conf.urls import url


from tamara_payment.views import TamaraPaymentView, TamaraPaymentCheckAvailabilityView

urlpatterns = [
    url("check-availability/", TamaraPaymentCheckAvailabilityView.as_view(), name="check-availability"),
    url(r"^$", TamaraPaymentView.as_view(), name="tamara-payment"),
]



import os

import requests
from rest_framework import status


class MockResponseMixin:
    @staticmethod
    def _get_response(file_name, extension="json"):
        dir_path = os.path.dirname(os.path.abspath(__file__))
        file_path = os.path.join(dir_path, "response/{}.{}".format(file_name, extension))
        with open(file_path) as f:
            return f.read()

    @staticmethod
    def _mock_response(status_code=status.HTTP_200_OK, content=None, headers=None):
        response = requests.models.Response()
        response.status_code = status_code
        response._content = content.encode() if isinstance(content, str) else content
        response.headers = headers or {}
        return response



import django
from django.conf import settings
from django.http import Http404
from django.template.response import TemplateResponse
from django.test import SimpleTestCase
from django.test import override_settings
from django.test.client import RequestFactory
from mock.mock import patch
from rest_framework import status

from tamara_payment.tests.mixins import MockResponseMixin

if not settings.configured:
    settings.configure()


@override_settings(
    PZ_SERVICE_CLASS="tamara_payment.commerce.dummy.Service",
    TAMARA_EXTENSION_URL="http://example.com",
    HASH_SECRET_KEY="cgDywMThqhuxBOEEnjhfgeFGxBJZJJLa6Xc3WpqKn"
)
class TestTamaraPaymentRedirectView(SimpleTestCase, MockResponseMixin):
    def setUp(self):
        from tamara_payment.views import TamaraPaymentView
        django.setup()
        self.view = TamaraPaymentView
        self.request_factory = RequestFactory()

    @override_settings(TAMARA_EXTENSION_URL=None)
    def test_none_tamara_extension_url(self):
        request = self.request_factory.get("/payment-gateway/tamara/")
        request.GET = {"sessionid": "test-session-id"}
        with self.assertRaises(Http404):
            response = self.view.as_view()(request)

    @patch("tamara_payment.commerce.dummy.Service.get")
    def test_none_pre_order(self, mock_get):
        response = self._mock_response(
            status_code=200,
            content={},
            headers={"Content-Type": "application/json"},
        )
        mock_get.return_value = response

        request = self.request_factory.get("/payment-gateway/tamara/")
        request.GET = {"sessionId": "test-session-id"}
        with self.assertRaises(Http404):
            self.view.as_view()(request)

    @patch("tamara_payment.commerce.dummy.Service.get")
    @patch("tamara_payment.commerce.checkout.CheckoutService.generate_hash")
    @patch("secrets.token_hex")
    def test_get(self, mock_token_hex, mock_generate_hash, mock_get):
        response = self._mock_response(
            status_code=200,
            content=self._get_response("orders_checkout_response"),
            headers={"Content-Type": "application/json"},
        )
        mock_get.return_value = response
        mock_generate_hash.return_value = "test-hash"
        mock_token_hex.return_value = "test-salt"

        request = self.request_factory.get("/payment-gateway/tamara/")
        request.GET = {"sessionId": "test-session-id"}
        response = self.view.as_view()(request)
        mock_generate_hash.assert_called_once_with("test-salt", "test-session-id")

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIsInstance(response, TemplateResponse)
        self.assertEqual(response.template_name, "tamara_payment.html")

        context = response.context_data
        self.assertIn("tamara_form", context)



import secrets
import hashlib
import json
import logging
from decimal import Decimal, ROUND_DOWN

from tamara_payment.commerce.data_models import RequestBody

from django.conf import settings
from django.http import Http404
from importlib import import_module


module, _class = settings.PZ_SERVICE_CLASS.rsplit(".", 1)
Service = getattr(import_module(module), _class)

logger = logging.getLogger(__name__)


class CheckoutService(Service):

    def get_pre_order_data(self, request):
        salt = self.generate_salt()
        session_id = request.GET.get("sessionId")
        
        hash_ = self.generate_hash(salt, session_id)
        request_body = self._get_order_data(request)

        return json.dumps({
            "hash": hash_,
            "salt": salt,
            "tax_amount": {"amount": request_body.total_tax_amount},
            "shipping_amount": {"amount": request_body.shipping_amount},
            "order_items": request_body.basket_items,
            "shipping_address": request_body.shipping_address,
            "billing_address": request_body.billing_address
        }, ensure_ascii=False)

    def generate_salt(self, length=0):
        salt = secrets.token_hex(10)
        return salt if not length else salt[:length]

    def generate_hash(self, salt, *args):
        hash_key = getattr(settings, "HASH_SECRET_KEY")
        hash_body = "|".join(args)
        return hashlib.sha512(
            f"{salt}|{hash_body}|{hash_key}".encode("utf-8")
        ).hexdigest()

    def _get_product_name(self, product_name):
        if not product_name:
            product_name = "none"
        return product_name if len(product_name) <= 255 else product_name[:255]
  
    def _validate_checkout_step(self, response):
        if "pre_order" not in response.data:
            raise Http404()
        
        if response.data["pre_order"].get("shipping_option") is None:
            raise Http404()
        
        if response.data["pre_order"].get("payment_option") is None:
            raise Http404()
    
    def _find_decimal_places(self, price):
        if '.' in price:
            return len(price.split('.')[1])
        return 0
    
    def _get_quantize_format(self, price_as_string):
        decimal_places = self._find_decimal_places(price_as_string)
        if decimal_places == 0:
            return Decimal(0)
        quantize_format = Decimal(f".{'0' * (decimal_places-1)}1")
        return quantize_format

    def _get_address(self, address_response):
        return {
            "city": address_response["city"]["name"],
            "country_code": address_response["country"]["code"].upper(),
            "first_name": address_response["first_name"],
            "last_name": address_response["last_name"],
            "line1": address_response["line"],
            "phone_number": address_response["phone_number"],
            "region": address_response["township"].get("name", ""),
        }

    def _get_order_data(self, request):
        response = self._retrieve_pre_order(request)
        self._validate_checkout_step(response=response)
        unpaid_amount = Decimal(response.data["pre_order"].get("unpaid_amount", 0))

        if unpaid_amount == Decimal(0):
            logger.info("Tamara Payment Unpaid amount is Zero")
            return []

        shipping_amount = Decimal(response.data["pre_order"]["shipping_amount"])
        shipping_amount = shipping_amount if shipping_amount < unpaid_amount else 0

        response_basket_items = response.data["pre_order"]["basket"]["basketitem_set"]
        quantize_format = self._get_quantize_format(response.data["pre_order"]["unpaid_amount"])
        total_product_amount = Decimal(response.data["pre_order"]["basket"]["total_product_amount"])
        unpaid_amount_without_shipping = unpaid_amount - shipping_amount
        remaining_amount = max(unpaid_amount_without_shipping - total_product_amount, 0)

        basket_items = []
        cumulative_amount = Decimal(0)
        total_tax_amount = Decimal(0)

        for index, item in enumerate(response_basket_items):
            basket_item_amount = Decimal(item["total_amount"])
            weight = basket_item_amount / total_product_amount
            amount = (remaining_amount * weight + basket_item_amount).quantize(quantize_format,
                                                                               ROUND_DOWN)
            cumulative_amount += amount

            if index == len(response_basket_items) - 1:
                # Adjust the amount for the last item to ensure the total matches unpaid amount
                delta = unpaid_amount_without_shipping - cumulative_amount
                amount = amount + delta

            tax_rate = Decimal(item.get("tax_rate", 0)) / 100

            total_tax_amount += (amount * tax_rate).quantize(quantize_format, ROUND_DOWN)
            basket_items.append({
                "name": self._get_product_name(item.get("product", {}).get("name")),
                "type": item.get("product", {}).get("category", {}).get("name"),
                "reference_id": item.get("pk"),
                "sku": item.get("product", {}).get("sku"),
                "quantity": item.get("quantity"),
                "total_amount": {
                    "amount": str(amount),
                }
            })

        return RequestBody(
            shipping_amount=str(shipping_amount),
            total_tax_amount=str(total_tax_amount),
            basket_items=basket_items,
            shipping_address=self._get_address(response.data["pre_order"]["shipping_address"]),
            billing_address=self._get_address(response.data["pre_order"]["billing_address"])
        )

    def _retrieve_pre_order(self, request):
        path = "/orders/checkout/?page=OrderNotePage"
        response = self.get(
            path, request=request, headers={"X-Requested-With": "XMLHttpRequest"}
        )
        return self.normalize_response(response)

    def check_availability_request_body(self, request):
        response = self._retrieve_pre_order(request)
        self._validate_checkout_step(response=response, required_page="")

        salt = self.generate_salt(length=10)

        # TODO: country code should be the merchant's country code
        country_code = response.data["pre_order"]["shipping_address"]["country"]["code"].upper()
        unpaid_amount = response.data["pre_order"].get("unpaid_amount")
        currency = response.data["pre_order"]["basket"]["basketitem_set"][0]["currency_type_label"]
        phone_number = (response.data["pre_order"]["user_phone_number"] or
                        response.data["pre_order"]["shipping_address"]["phone_number"])

        return {
            "salt": salt,
            "hash": self.generate_hash(salt, country_code, phone_number),
            "country": country_code,
            "phone_number": phone_number,
            "order_value": {
                "amount": unpaid_amount,
                "currency": currency
            }
        }



import logging

from django.http import Http404, JsonResponse
from django.views.generic import View
from django.template.response import TemplateResponse

from tamara_payment.commerce.checkout import CheckoutService
from tamara_payment.forms import TamaraForm
from tamara_payment.extensions import TamaraPayment


logger = logging.getLogger(__name__)


class TamaraPaymentView(View):
    checkout_service = CheckoutService()
    _tamara_payment = TamaraPayment()
    http_method_names = ["get"]

    def get(self, request):
        session_id = request.GET.get("sessionId")
        if not session_id:
            logging.exception("Missing sessionId")
            raise Http404

        data = self.checkout_service.get_pre_order_data(request)
        tamara_form = TamaraForm(
            initial={"data": data}
        )

        return TemplateResponse(
            request=request,
            template="tamara_payment.html",
            context={
                "action_url": f"{self._tamara_payment.url}/form-page?sessionId={session_id}",
                "action_method": "POST",
                "tamara_form": tamara_form,
            },
        )


class TamaraPaymentCheckAvailabilityView(View):
    checkout_service = CheckoutService()
    http_method_names = ["get"]
    _tamara_payment = TamaraPayment()

    def get(self, request):
        request_body = self.checkout_service.check_availability_request_body(request)
        is_available = self._tamara_payment.check_availability(request_body)
        return JsonResponse({"is_available": is_available})



from decimal import Decimal
import json

from unittest.mock import Mock
from django.http import Http404
from django.conf import settings
from django.test import SimpleTestCase
from django.test import override_settings
from django.test.client import RequestFactory
from mock.mock import patch
from rest_framework import status

from tamara_payment.tests.mixins import MockResponseMixin

try:
    settings.configure()
except RuntimeError:
    pass


@override_settings(
    HASH_SECRET_KEY="test-hash-secret-key",
    PZ_SERVICE_CLASS="tamara_payment.commerce.dummy.Service",
)
class TestCheckoutService(SimpleTestCase, MockResponseMixin):
    def setUp(self):
        from tamara_payment.commerce.checkout import CheckoutService

        self.service = CheckoutService()
        self.request_factory = RequestFactory()

    @patch("tamara_payment.commerce.dummy.Service.get")
    @patch("tamara_payment.commerce.checkout.CheckoutService.generate_hash")
    @patch("tamara_payment.commerce.checkout.CheckoutService.generate_salt")
    def test_get_data(self, mock_generate_salt, mock_generate_hash, mock_get):
        mock_generate_hash.return_value = "test-hash"
        mock_generate_salt.return_value = "test-salt"
        mocked_response = self._mock_response(
            status_code=200,
            content=self._get_response("orders_checkout_response"),
            headers={"Content-Type": "application/json"},
        )
        mock_get.return_value = mocked_response

        request = self.request_factory.get("/payment-gateway/tamara/")
        basket_data = self.service.get_pre_order_data(request)
        basket_data = json.loads(basket_data)

        self.assertEqual(basket_data["salt"], "test-salt")
        self.assertEqual(basket_data["hash"], "test-hash")

        self.assertEqual(basket_data["tax_amount"], {"amount": "37.92"})
        self.assertEqual(basket_data["shipping_amount"], {"amount": "33.00"})
        self.assertEqual(basket_data["shipping_address"], {
            "city": "İstanbul",
            "country_code": "TR",
            "first_name": "akinon",
            "last_name": "akinon",
            "line1": "YTÜ-Davutpaşa Kampüsü Teknopark Bölgesi B2 Blok D:Kat:2, No: 417",
            "phone_number": "0555555555",
            "region": "ESENLER"
        })
        self.assertEqual(basket_data["billing_address"], {
            "city": "İstanbul",
            "country_code": "TR",
            "first_name": "akinon",
            "last_name": "akinon",
            "line1": "YTÜ-Davutpaşa Kampüsü Teknopark Bölgesi B2 Blok D:Kat:2, No: 417",
            "phone_number": "0555555555",
            "region": "ESENLER"
        })

        basket_items = [
            {
                "name": "Petıt / 110x170cm Dijital Baskılı Halı",
                "type": "Halı",
                "reference_id": 923,
                "sku": "2672881033026",
                "quantity": 4,
                "total_amount": {
                    "amount": "224.76"
                }
            },
            {
                "name": "50cm Bombeli Saat Desen 13",
                "type": "Duvar Saatleri",
                "reference_id": 922,
                "sku": "2672880349036",
                "quantity": 2,
                "total_amount": {
                    "amount": "79.84"
                }
            },
            {
                "name": "Demet Lavanta Çiçek 62cm",
                "type": "Yapay Çiçek",
                "reference_id": 921,
                "sku": "2672881041106",
                "quantity": 3,
                "total_amount": {
                    "amount": "30.96"
                }
            }
        ]
        self.assertEqual(basket_data["order_items"],  basket_items)

    @patch("tamara_payment.commerce.dummy.Service.get")
    def test_retrieve_pre_oder(self, mock_get):
        mocked_response = self._mock_response(
            status_code=200,
            content=self._get_response("orders_checkout_response"),
            headers={"Content-Type": "application/json"},
        )
        mock_get.return_value = mocked_response

        request = self.request_factory.get("/payment-gateway/tamara/")
        response = self.service._retrieve_pre_order(request)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn("pre_order", response.data)

    @patch("tamara_payment.commerce.dummy.Service.get")
    def test_get_basket_items(self, mock_get):
        mocked_response = self._mock_response(
            status_code=200,
            content=self._get_response("orders_checkout_response"),
            headers={"Content-Type": "application/json"},
        )
        mock_get.return_value = mocked_response

        request = self.request_factory.get("/payment-gateway/tamara/")
        request_body = self.service._get_order_data(request)

        self.assertEqual(request_body.total_tax_amount, "37.92")
        self.assertEqual(request_body.shipping_amount, "33.00")

        expected_basket_item = [
            {
                "name": "Petıt / 110x170cm Dijital Baskılı Halı",
                "type": "Halı",
                "reference_id": 923,
                "sku": "2672881033026",
                "quantity": 4,
                "total_amount": {
                    "amount": "224.76"
                }
            },
            {
                "name": "50cm Bombeli Saat Desen 13",
                "type": "Duvar Saatleri",
                "reference_id": 922,
                "sku": "2672880349036",
                "quantity": 2,
                "total_amount": {
                    "amount": "79.84"
                }
            },
            {
                "name": "Demet Lavanta Çiçek 62cm",
                "type": "Yapay Çiçek",
                "reference_id": 921,
                "sku": "2672881041106",
                "quantity": 3,
                "total_amount": {
                    "amount": "30.96"
                }
            }
        ]
        self.assertEqual(
            request_body.basket_items, expected_basket_item
        )
        self.assertNotEqual(request_body.shipping_address, None)
        self.assertNotEqual(request_body.billing_address, None)

    @patch("hashlib.sha512")
    def test_get_hash(self, mock_sha512):
        session_id = "test-session-id"
        self.service.generate_hash("test-salt", session_id)
        mock_sha512.assert_called_once_with(
            "test-salt|test-session-id|test-hash-secret-key".encode("utf-8")
        )

    @patch("secrets.token_hex")
    def test_generate_salt(self, mock_token_hex):
        self.service.generate_salt()
        mock_token_hex.assert_called_once()
        
    def test_get_product_name(self):
        product_name = None
        self.assertEqual(self.service._get_product_name(product_name), "none")
        
        product_name = "t" * 254
        self.assertEqual(self.service._get_product_name(product_name), product_name)
             
        product_name = "t" * 255
        self.assertEqual(self.service._get_product_name(product_name), product_name)
        
        product_name = "t" * 256
        self.assertEqual(self.service._get_product_name(product_name), "t" * 255)
        
    def test__find_decimal_places(self):
        self.assertEqual(self.service._find_decimal_places("10.000"), 3)
        self.assertEqual(self.service._find_decimal_places("10.00"), 2)
        self.assertEqual(self.service._find_decimal_places("10.0"), 1)
        self.assertEqual(self.service._find_decimal_places("10"), 0)
        
    def test__get_quantize_format(self):
        self.assertEqual(self.service._get_quantize_format("10.000"), Decimal(".001"))
        self.assertEqual(self.service._get_quantize_format("10.00"), Decimal(".01"))
        self.assertEqual(self.service._get_quantize_format("10.0"), Decimal(".1"))
        self.assertEqual(self.service._get_quantize_format("10"), Decimal("0"))
    
    def test__validate_checkout_step(self):
        mock_resp = Mock()
        mock_resp.data = {}
        
        with self.assertRaises(Http404):
            self.service._validate_checkout_step(mock_resp)
        
        mock_resp.data = {"pre_order": {"shipping_option": None}}
        with self.assertRaises(Http404):
            self.service._validate_checkout_step(mock_resp)

        mock_resp.data = {
            "context_list": [{"page_name": "ShippingOptionSelectionPage"}],
            "pre_order": {
                "shipping_option": {
                    "pk": 2,
                    "name": "Yurtici Kargo",
                    "slug": "yurtici",
                    "logo": None,
                    "shipping_amount": "9.99",
                    "description": None,
                    "kwargs": {}
                        }
                }
        }
        with self.assertRaises(Http404):
            self.service._validate_checkout_step(mock_resp)
            
        mock_resp.data = {
            "context_list": [{"page_name": "PaymentOptionSelectionPage"}],
            "pre_order": {
                "shipping_option": {
                    "pk": 2,
                    "name": "Yurtici Kargo",
                    "slug": "yurtici",
                    "logo": None,
                    "shipping_amount": "9.99",
                    "description": None,
                    "kwargs": {}
                },
                 "unpaid_amount": 10,
                "payment_option": {
                "slug": "tamara",
                },
            }
        }
        self.service._validate_checkout_step(mock_resp)

    def test_shipping_address(self):
        _shipping_address = self.service._get_address({
              "pk": 50,
              "email": "akinon@akinon.com",
              "phone_number": "0555555555",
              "first_name": "akinon",
              "last_name": "akinon",
              "country": {
                  "pk": 1,
                  "name": "Türkiye",
                  "code": "tr"
              },
              "city": {
                  "pk": 47,
                  "name": "İstanbul",
                  "country": 1
              },
              "line": "YTÜ-Davutpaşa Kampüsü Teknopark Bölgesi B2 Blok D:Kat:2, No: 417",
              "title": "ofis",
              "township": {
                  "pk": 209,
                  "name": "ESENLER",
                  "city": 47
              },
              "district": {
                  "pk": 2194,
                  "name": "ÇİFTEHAVUZLAR/ESENLER",
                  "city": 47,
                  "township": 209
              },
              "postcode": "34220",
              "notes": None,
              "company_name": "",
              "tax_office": "",
              "tax_no": "",
              "e_bill_taxpayer": False,
              "is_corporate": False,
              "primary": False,
              "identity_number": None,
              "extra_field": None
        })
        self.assertEqual(_shipping_address, {
                "city": "İstanbul",
                "country_code": "TR",
                "first_name": "akinon",
                "last_name": "akinon",
                "line1": "YTÜ-Davutpaşa Kampüsü Teknopark Bölgesi B2 Blok D:Kat:2, No: 417",
                "phone_number": "0555555555",
                "region": "ESENLER"
            }
        )



from django.http import Http404
from json import JSONDecodeError
from rest_framework import status


class Service:
    def get(self):
        pass

    def normalize_response(self, response):
        from django.http.response import HttpResponseRedirect
        from rest_framework.response import Response

        if response.status_code == status.HTTP_404_NOT_FOUND:
            raise Http404()

        if response.is_redirect:
            url = response.headers["location"]
            if url.rfind("format") > 0:
                url = url[:url.rfind("format")]
            return HttpResponseRedirect(url, status=response.status_code)

        acceptable_statuses = [status.HTTP_429_TOO_MANY_REQUESTS]

        if response.status_code > status.HTTP_406_NOT_ACCEPTABLE and \
                response.status_code not in acceptable_statuses:
            response.raise_for_status()

        if response.status_code == status.HTTP_204_NO_CONTENT:
            return Response(status=status.HTTP_204_NO_CONTENT)

        data = self.normalize(response)

        return Response(data=data, status=response.status_code)

    @staticmethod
    def normalize(response):
        data = {}
        if response.content:
            try:
                data = response.json()
            except JSONDecodeError as exc:
                data = {"content": response.content}
        return data



import setuptools


def readme():
    with open("readme.md", "r") as f:
        return f.read()


setuptools.setup(
    name="tamara_payment",
    version="1.4.0",
    author="Akinon",
    author_email="dev@akinon.com",
    description="A library to provide payment gateway for Tamara Payment",
    long_description=readme(),
    long_description_content_type="text/markdown",
    url="https://bitbucket.org/akinonteam/tamara-payment/",
    packages=setuptools.find_packages(exclude=["tests", "tests.*", "dummy.*"]),
    zip_safe=False,
    install_requires=[
        "Django>=2.2.9,<4.0",
        "requests",
        "djangorestframework>=3.11.0,<3.15",
        "mock==4.0.3",
    ],
    include_package_data=True,
    package_data={"tamara_payment": ["templates/*"]},
)
